@@ -1,16 +1,16 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-1997, Patrick Powell, San Diego, CA
- *     papowell@sdsu.edu
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
+ *     papowell@lprng.com
  * See LICENSE for conditions of use.
- *
- ***************************************************************************
- * MODULE: errorcode.h
- * PURPOSE: filter error return codes
- * "errorcodes.h,v 3.3 1998/03/29 18:33:01 papowell Exp"
- **************************************************************************/
+ * $Id: errorcodes.h,v 1.27 2002/04/01 17:54:57 papowell Exp $
+ ***************************************************************************/
 
+
+
+#ifndef _ERRORCODES_H_
+#define _ERRORCODES_H_ 1
 /*
  * filter return codes and job status codes
  * - exit status of the filter process 
@@ -23,10 +23,18 @@
 #define JFAIL    32    /* failed - retry later */
 #define JABORT   33    /* aborted - do not try again, but keep job */
 #define JREMOVE  34    /* failed - remove job */
-#define JACTIVE  35    /* active server - try later */
-#define JIGNORE  36    /* ignore this job - not used! */
 #define JHOLD    37    /* hold this job */
 #define JNOSPOOL 38    /* no spooling to this queue */
 #define JNOPRINT 39    /* no printing from this queue  */
 #define JSIGNAL  40    /* killed by unrecognized signal */
-#define JFAILNORETRY  41 /* no retry on failure */
+#define JFAILNORETRY 41 /* no retry on failure */
+#define JSUSP    42		/* process suspended successfully */
+#define JTIMEOUT 43		/* timeout */
+#define JWRERR   44		/* write error */
+#define JRDERR   45		/* read error  */
+#define JCHILD   46		/* no children */
+#define JNOWAIT  47		/* no wait status */
+
+/* PROTOTYPES */
+
+#endif
