@@ -1,11 +1,16 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-1997, Patrick Powell, San Diego, CA
- *     papowell@sdsu.edu
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
+ *     papowell@lprng.com
  * See LICENSE for conditions of use.
- *
- ***************************************************************************
+ * $Id: portable.h,v 1.27 2002/04/01 17:54:58 papowell Exp $
+ ***************************************************************************/
+
+#ifndef _PLP_PORTABLE_H
+#define _PLP_PORTABLE_H 1
+
+/***************************************************************************
  * MODULE: portable.h
  * PURPOSE:
  * The configure program generates config.h,  which defines various
@@ -25,8 +30,11 @@
  * portable.h,v 3.14 1998/03/24 02:43:22 papowell Exp
  **************************************************************************/
 
-#ifndef _PLP_PORTABLE_H
-#define _PLP_PORTABLE_H 1
+#if !defined(EXTERN)
+#define EXTERN extern
+#define DEFINE(X) 
+#undef DEFS
+#endif
 
 #ifndef __STDC__
 LPRng requires ANSI Standard C compiler
@@ -34,6 +42,10 @@ LPRng requires ANSI Standard C compiler
 
 #include "config.h"
 
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+
 /*************************************************************************
  * ARGH: some things that "configure" can't get right.
 *************************************************************************/
@@ -54,7 +66,7 @@ LPRng requires ANSI Standard C compiler
  * This appears to be historical.
  *************************************************************************/
 #ifdef apollo
-# define IS_APOLLO
+# define IS_APOLLO OSVERSION
 /* #undef __STDC__ */
 /* # define CONFLICTING_PROTOS */
 #endif
@@ -66,7 +78,7 @@ LPRng requires ANSI Standard C compiler
  * Take a chance on using the standard calls
  *************************************************************************/
 #ifdef ultrix
-# define IS_ULTRIX
+# define IS_ULTRIX OSVERSION
 #endif
 
 
@@ -74,7 +86,7 @@ LPRng requires ANSI Standard C compiler
  * AIX.
  *************************************************************************/
 #ifdef _AIX32 
-# define IS_AIX32
+# define IS_AIX32 OSVERSION
 #endif
 
 /*************************************************************************
@@ -84,6 +96,23 @@ LPRng requires ANSI Standard C compiler
 #if defined(sun)
 #endif
 
+/*************************************************************************
+ * SCO OpenServer 5.0.5
+ *************************************************************************/
+/* normal include files do not define MAXPATHLEN - rather PATHSIZE in 
+   sys/param.h */
+#ifdef sco
+#ifndef MAXPATHLEN
+#define MAXPATHLEN	PATHSIZE
+#endif
+/* SCO doesn't define the S_ISSOCK POSIX macro to use in testing the 
+   stat.st_mode structure member  - it appears as though a socket has
+   st_mode = 0020000 (same as character special) */
+#ifndef S_ISSOCK
+#define S_IFSOCK	0020000
+#define S_ISSOCK(m)	(((m) & S_IFMT) == S_IFSOCK)
+#endif
+#endif
 
 /*************************************************************************
  * Cray
@@ -104,44 +133,36 @@ LPRng requires ANSI Standard C compiler
 
 /*************************************************************************/
 #if defined(NeXT)
-# define IS_NEXT
+# define IS_NEXT OSVERSION
 # define __STRICT_BSD__
 #endif
 
 /*************************************************************************/
 #if defined(__sgi) && defined(_SYSTYPE_SVR4)
-# define IS_IRIX5
+# define IS_IRIX5 OSVERSION
 #endif
 
 /*************************************************************************/
 #if defined(__sgi) && defined(_SYSTYPE_SYSV)
-#define IS_IRIX4
+#define IS_IRIX4 OSVERSION
 #endif
 
 /*************************************************************************/
 #if defined(__linux__) || defined (__linux) || defined (LINUX)
-# define IS_LINUX
+# define IS_LINUX OSVERSION
 #endif
 
-
-/*************************************************************************/
-#if defined(__hpux) || defined(_HPUX_SOURCE)
-# define IS_HPUX
-# undef _HPUX_SOURCE
-# define _HPUX_SOURCE 1
-#endif
-  
 /*************************************************************************/
 
 #if defined(__convex__) /* Convex OS 11.0 - from w_stef */
-# define IS_CONVEX
+# define IS_CONVEX OSVERSION
 # define LPASS8 (L004000>>16)
 #endif
 
 /*************************************************************************/
 
 #ifdef _AUX_SOURCE
-# define IS_AUX
+# define IS_AUX OSVERSION
 # define _POSIX_SOURCE
 
 # undef SETPROCTITLE
@@ -151,7 +172,7 @@ LPRng requires ANSI Standard C compiler
 /*************************************************************************/
 
 #if defined(SNI) && defined(sinix)
-# define IS_SINIX
+# define IS_SINIX OSVERSION
 #endif
 
 
@@ -170,7 +191,7 @@ LPRng requires ANSI Standard C compiler
 
 /*************************************************************************/
 #if defined(__bsdi__)
-# define IS_BSDI
+# define IS_BSDI OSVERSION
 #endif
 
 /*************************************************************************/
@@ -264,9 +285,13 @@ LPRng requires ANSI Standard C compiler
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <sys/un.h>
 
 #include <sys/stat.h>
 #include <pwd.h>
+#if defined(HAVE_SYS_SIGNAL_H)
+#  include <sys/signal.h>
+#endif
 #include <signal.h>
 #include <sys/wait.h>
 #include <ctype.h>
@@ -290,6 +315,9 @@ LPRng requires ANSI Standard C compiler
 #if !defined(HAVE_STRNCASECMP)
  int strncasecmp (const char *s1, const char *s2, int len );
 #endif
+#if !defined(HAVE_STRCASECMP_DEF)
+ int strcasecmp (const char *s1, const char *s2 );
+#endif
 
 
 /*********************************************************************
@@ -327,10 +355,10 @@ typedef struct dirent plp_dir_t;
  * Patrick Powell Thu Apr  6 07:47:54 PDT 1995
  *********************************************************************/
 
-#ifdef HAVE_MALLOC_H
-# include <malloc.h>
-#else
-# if !defined(HAVE_STDLIB_H)
+#if !defined(HAVE_STDLIB_H)
+# ifdef HAVE_MALLOC_H
+#   include <malloc.h>
+# else
    void *malloc(size_t);
    void free(void *);
 # endif
@@ -352,9 +380,9 @@ typedef struct dirent plp_dir_t;
 # include <time.h>
 #else
 # ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#else
-# include <time.h>
+#   include <sys/time.h>
+# else
+#   include <time.h>
 # endif
 #endif
 
@@ -608,22 +636,16 @@ XX ** NO VARARGS ** XX
  * different systems
  */
 
-#ifndef O_NONBLOCK
-#define O_NONBLOCK 0
-#endif
-
-#ifndef O_NOCTTY
-#define O_NOCTTY 0
-#endif
-
-
 #define NONBLOCK (O_NDELAY|O_NONBLOCK)
-#ifdef IS_HPUX
-#undef NONBLOCK
-#define NONBLOCK (O_NONBLOCK)
+#if defined(HPUX) && HPUX<110
+#  undef NONBLOCK
+#  define NONBLOCK (O_NONBLOCK)
+#  undef FD_SET_FIX
+#  define FD_SET_FIX(X) (int *)
 #endif
 
 
+
 /*********************************************************************
  * AIX systems need this
  *********************************************************************/
@@ -637,13 +659,21 @@ XX ** NO VARARGS ** XX
  **********************************************************************/
 #ifdef HAVE_SIGPROCMASK
 /* a signal set */
-typedef sigset_t plp_block_mask;
+#define plp_block_mask sigset_t
 #else
 /* an integer */
-typedef int plp_block_mask;
+#define plp_block_mask int
 #endif
 
-#define FD_SET_FIX(X) X
+/**********************************************************************
+ *  Select() problems
+ **********************************************************************/
+#ifdef HAVE_SELECT_H
+#include <select.h>
+#endif
+#if !defined(FD_SET_FIX)
+# define FD_SET_FIX(X) X
+#endif
 
 /**********************************************************************
  * IPV6 and newer versions
@@ -655,6 +685,44 @@ int inet_pton( int family, const char *strptr, void *addr );
 const char *inet_ntop( int family, const void *addr, char *strptr, size_t len );
 #endif
 
+
+/*****************************************************
+ * Internationalisation of messages, using GNU gettext
+ *****************************************************/
+
+#if HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(Text) gettext(Text)
+# ifdef gettext_noop
+#  define N_(Text) gettext_noop(Text)
+# else
+#  define N_(Text) Text
+# endif
+#else
+# define _(Text) Text
+# define N_(Text) Text
+# define textdomain(Domain)
+# define bindtextdomain(Package, Directory)
+#endif
+
+
+/**********************************************************************
+ *  Cygwin Definitions
+ **********************************************************************/
+#ifdef __CYGWIN__
+#define ROOTUID 18
+#else
+#define ROOTUID 0
+#endif
+
+#ifndef HAVE_FLOCK_DEF
+extern int flock( int fd, int operation );
+#endif
+
 /**********************************************************************
  *  SUNOS Definitions
  **********************************************************************/
@@ -686,7 +754,7 @@ extern int lockf(int fd, int cmd, long size );
 /*extern int lseek(int fd, off_t pos, int how ); */
 extern int lstat(const char *path, struct stat *buf );
 #define memmove(dest,src,len) bcopy(src,dest,len)
-extern void bcopy(char *src,char *dest,int len);
+extern void bcopy(const void *src,void *dest,size_t len);
 extern int mkstemp(char *s );
 extern int openlog( const char *ident, int logopt, int facility );
 extern int perror(const char *);
@@ -704,7 +772,13 @@ extern int stat(const char *path, struct stat *buf );
 extern int strcasecmp( const char *, const char * );
 extern char *strerror( int );
 extern int strncasecmp( const char *, const char *, int n );
-extern int long strtol( char *str, char **ptr, int base );
+extern long strtol( const char *str, char **ptr, int base );
+extern double strtod( const char *str, char **ptr );
+extern int shutdown( int sock, int how );
+extern int gettimeofday(struct timeval *tp, struct timezone *tzp);
+extern int getrlimit(int resource, struct rlimit *rlp);
+extern char * sbrk(int incr);
+extern int fchmod(int fd, int mode);
 extern int strftime(char *buf, int bufsize, const char *fmt, struct tm *tm);
 extern void syslog(int, const char *, ...);
 extern int system( const char *str );
@@ -714,16 +788,9 @@ extern int toupper( int );
 extern int tputs( const char *cp, int affcnt, int (*outc)() );
 extern int vfprintf(FILE *, const char *, ...);
 extern int vprintf(FILE *, const char *, va_list ap);
-/* surprisingly, there are no declarations for random on Sun stuff */
 #endif
 
 
-#ifndef STDLIB_DEFINES_RANDOM
- /* surprisingly, sometimes there are no declarations for random */
-  extern long random(void);
-  extern void srandom(unsigned int seed);
-#endif
-
 #ifdef SOLARIS
 extern int setreuid( uid_t ruid, uid_t euid );
 extern int mkstemp(char *s );
@@ -732,47 +799,24 @@ extern int getdtablesize(void);
 #endif
 #endif
 
-#ifdef HPUX
+#if !defined(HAVE_SYSLOG_DEF)
 extern void syslog(int, const char *, ...);
+#endif
 #if !defined(HAVE_OPENLOG_DEF)
 extern int openlog( const char *ident, int logopt, int facility );
 #endif
-#undef FD_SET_FIX
-#define FD_SET_FIX(X) (int *)
-#endif
 
 #ifdef IS_AIX32
-#if !defined(HAVE_OPENLOG_DEF)
-void openlog(const char *, int, int);
-void syslog(int, const char *, ...);
-#endif
 extern int seteuid(uid_t);
 #endif
 
 
 /* IPV6 structures define */
 
-#if !defined(AF_INET6)
-#  define AF_INET6 24
-#endif
-
-#if defined(IN_ADDR6)
-# define in6_addr in_addr6
-# define IN6_ADDR
-#endif
-
-#if !defined(IN6_ADDR)
-# define IN6_ADDR
-# define SIN6_LEN 16
-
-struct in6_addr { unsigned char s6_addr[SIN6_LEN]; };
-struct sockaddr_in6 {
-	/* unsigned char sin6_len; */
-	unsigned char sin6_family;
-	unsigned short sin6_port;
-	unsigned int sin6_flowinfo;
-	struct in6_addr sin6_addr;
-};
+#if defined(AF_INET6)
+# if defined(IN_ADDR6)
+#  define in6_addr in_addr6
+# endif
 #endif
 
 #if defined(HAVE_ARPA_NAMESER_H)
@@ -782,4 +826,36 @@ struct sockaddr_in6 {
 # include <resolv.h>
 #endif
 
-#endif	/* PLP_PORTABLE_H */
+#ifdef HAVE_INNETGR
+#if !defined(HAVE_INNETGR_DEF)
+extern int innetgr(const char *netgroup,
+    const char *machine, const char *user, const char *domain);
+#endif
+#endif
+
+
+#define Cast_int_to_voidstar(v) ((void *)(long)(v))
+#define Cast_ptr_to_int(v) ((int)(long)(v))
+#define Cast_ptr_to_long(v) ((long)(v))
+
+/* for testing, set -Wall -Wformat and then make */
+
+#if defined(FORMAT_TEST)
+# define FPRINTF fprintf
+# define PRINTF printf
+# define STDOUT stdout
+# define STDERR stderr
+# define SNPRINTF(X,Y) printf(
+# define VSNPRINTF(X,Y) vprintf(
+# define SETSTATUS(X) printf(
+#else
+# define FPRINTF safefprintf
+# define PRINTF safeprintf
+# define STDOUT 1
+# define STDERR 2
+# define SNPRINTF(X,Y) plp_snprintf(X,Y,
+# define VSNPRINTF(X,Y) plp_vsnprintf(X,Y,
+# define SETSTATUS(X) setstatus(X,
+#endif
+
+#endif
